@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HUNCHENTOOT; Base: 10 -*-
 ;;; $Header: /usr/local/cvsrep/hunchentoot/cookie.lisp,v 1.8 2008/02/13 16:02:17 edi Exp $
 
-;;; Copyright (c) 2004-2008, Dr. Edmund Weitz.  All rights reserved.
+;;; Copyright (c) 2004-2009, Dr. Edmund Weitz.  All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -71,7 +71,7 @@ well. See <http://msdn2.microsoft.com/en-us/library/ms533046.aspx>."))
   "Ensure COOKIE has a correct slot-value for NAME."
   (let ((name (getf init-args :name)))
     (unless (http-token-p name)
-      (error "~S is not a legal name for a cookie." name)))
+      (parameter-error "~S is not a legal name for a cookie." name)))
   (call-next-method))
 
 (defun set-cookie* (cookie &optional (reply *reply*))
