@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HUNCHENTOOT; Base: 10 -*-
 ;;; $Header: /usr/local/cvsrep/hunchentoot/misc.lisp,v 1.17 2008/03/17 11:40:25 edi Exp $
 
-;;; Copyright (c) 2004-2008, Dr. Edmund Weitz. All rights reserved.
+;;; Copyright (c) 2004-2009, Dr. Edmund Weitz. All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -51,9 +51,11 @@ URL.  Scanners are memoized in SCANNER-HASH once they are created."
                     #\=
                     (:greedy-repetition 0 nil (:inverted-char-class #\&))
                     #\&))))))
-  (defun add-cookie-value-to-url (url &key (cookie-name *session-cookie-name*)
-                                           (value (session-cookie-value))
-                                           (replace-ampersands-p t))
+  (defun add-cookie-value-to-url (url &key
+                                      (cookie-name (session-cookie-name *acceptor*))
+                                      (value (when-let (session (session *request*))
+                                               (session-cookie-value session)))
+                                      (replace-ampersands-p t))
     "Removes all GET parameters named COOKIE-NAME from URL and then
 adds a new GET parameter with the name COOKIE-NAME and the value
 VALUE.  If REPLACE-AMPERSANDS-P is true all literal ampersands in URL
@@ -72,8 +74,10 @@ are replaced with '&amp;'. The resulting URL is returned."
       (setq url (regex-replace-all "&" url "&amp;")))
     url))
 
-(defun maybe-rewrite-urls-for-session (html &key (cookie-name *session-cookie-name*)
-                                                 (value (session-cookie-value)))
+(defun maybe-rewrite-urls-for-session (html &key
+                                            (cookie-name (session-cookie-name *acceptor*))
+                                            (value (when-let (session (session *request*))
+                                                     (session-cookie-value session))))
   "Rewrites the HTML page HTML such that the name/value pair
 COOKIE-NAME/COOKIE-VALUE is inserted if the client hasn't sent a
 cookie of the same name but only if *REWRITE-FOR-SESSION-URLS* is
@@ -91,14 +95,6 @@ true.  See the docs for URL-REWRITE:REWRITE-URLS."
                                           :cookie-name cookie-name
                                           :value value))))))))
 
-(defmethod dispatch-request (dispatch-table)
-  "Dispatches *REQUEST* based upon rules in the DISPATCH-TABLE.
-This method provides the default Hunchentoot behavior."
-  (loop for dispatcher in dispatch-table
-        for action = (funcall dispatcher *request*)
-        when action return (funcall action)
-        finally (setf (return-code *reply*) +http-not-found+)))
-
 (defun default-dispatcher (request)
   "Default dispatch function which handles every request with the
 function stored in *DEFAULT-HANDLER*."
@@ -108,51 +104,58 @@ function stored in *DEFAULT-HANDLER*."
 (defun default-handler ()
   "The handler that is supposed to serve the request if no other
 handler is called."
-  (log-message :info "Default handler called for script ~A" (script-name))
+  (log-message :info "Default handler called for script ~A" (script-name*))
   (format nil "<html><head><title>Hunchentoot</title></head><body><h2>Hunchentoot Default Page</h2><p>This is the Hunchentoot default page. You're most likely seeing it because the server administrator hasn't set up a custom default page yet.</p><p>Hunchentoot is a web server written in <a href='http://www.lisp.org/'>Common Lisp</a>.  More info about Hunchentoot can be found at <a href='http://weitz.de/hunchentoot/'>http://weitz.de/hunchentoot/</a>.</p></p><p><hr>~A</p></body></html>"
           (address-string)))
 
-(defun create-prefix-dispatcher (prefix page-function)
-  "Creates a dispatch function which will dispatch to the
-function denoted by PAGE-FUNCTION if the file name of the current
-request starts with the string PREFIX."
+(defun create-prefix-dispatcher (prefix handler)
+  "Creates a request dispatch function which will dispatch to the
+function denoted by HANDLER if the file name of the current request
+starts with the string PREFIX."
   (lambda (request)
     (let ((mismatch (mismatch (script-name request) prefix
                               :test #'char=)))
       (and (or (null mismatch)
                (>= mismatch (length prefix)))
-           page-function))))
+           handler))))
 
-(defun create-regex-dispatcher (regex page-function)
-  "Creates a dispatch function which will dispatch to the
-function denoted by PAGE-FUNCTION if the file name of the current
-request matches the CL-PPCRE regular expression REGEX."
+(defun create-regex-dispatcher (regex handler)
+  "Creates a request dispatch function which will dispatch to the
+function denoted by HANDLER if the file name of the current request
+matches the CL-PPCRE regular expression REGEX."
   (let ((scanner (create-scanner regex)))
     (lambda (request)
       (and (scan scanner (script-name request))
-           page-function))))
+           handler))))
+
+(defun abort-request-handler (&optional result)
+  "This function can be called by a request handler at any time to
+immediately abort handling the request.  This works as if the handler
+had returned RESULT.  See the source code of REDIRECT for an example."
+  (throw 'handler-done result))
 
 (defun handle-static-file (path &optional content-type)
   "A function which acts like a Hunchentoot handler for the file
-denoted by PATH.  Send a content type header corresponding to
-CONTENT-TYPE or \(if that is NIL) tries to determine the content
-type via the file's suffix."
-  (unless (and (fad:file-exists-p path)
-               (not (fad:directory-exists-p path)))
-    ;; does not exist
-    (setf (return-code) +http-not-found+)
-    (throw 'handler-done nil))
+denoted by PATH.  Sends a content type header corresponding to
+CONTENT-TYPE or \(if that is NIL) tries to determine the content type
+via the file's suffix."
+  (when (or (wild-pathname-p path)
+            (not (fad:file-exists-p path))
+            (fad:directory-exists-p path))
+    ;; file does not exist
+    (setf (return-code*) +http-not-found+)
+    (abort-request-handler))
   (let ((time (or (file-write-date path) (get-universal-time))))
-    (setf (content-type) (or content-type
-                             (mime-type path)
-                             "application/octet-stream"))
+    (setf (content-type*) (or content-type
+                              (mime-type path)
+                              "application/octet-stream"))
     (handle-if-modified-since time)
     (with-open-file (file path
                      :direction :input
                      :element-type 'octet
                      :if-does-not-exist nil)
-      (setf (header-out "Last-Modified") (rfc-1123-date time)
-            (content-length) (file-length file))
+      (setf (header-out :last-modified) (rfc-1123-date time)
+            (content-length*) (file-length file))
       (let ((out (send-headers)))
         #+:clisp
         (setf (flexi-stream-element-type *hunchentoot-stream*) 'octet)
@@ -163,10 +166,10 @@ type via the file's suffix."
                  (finish-output out))))))
 
 (defun create-static-file-dispatcher-and-handler (uri path &optional content-type)
-  "Creates and returns a dispatch function which will dispatch to a
-handler function which emits the file denoted by the pathname
+  "Creates and returns a request dispatch function which will dispatch
+to a handler function which emits the file denoted by the pathname
 designator PATH with content type CONTENT-TYPE if the SCRIPT-NAME of
-the request matches the string URI.  If CONTENT-TYPE is NIL tries to
+the request matches the string URI.  If CONTENT-TYPE is NIL, tries to
 determine the content type via the file's suffix."
   ;; the dispatcher
   (lambda (request)
@@ -190,12 +193,12 @@ it'll be the content type used for all files in the folder."
   (unless (and (stringp uri-prefix)
                (plusp (length uri-prefix))
                (char= (char uri-prefix (1- (length uri-prefix))) #\/))
-    (error "~S must be string ending with a slash." uri-prefix))
+    (parameter-error "~S must be string ending with a slash." uri-prefix))
   (when (or (pathname-name base-path)
             (pathname-type base-path))
-    (error "~S is supposed to denote a directory." base-path))
+    (parameter-error "~S is supposed to denote a directory." base-path))
   (flet ((handler ()
-           (let* ((script-name (url-decode (script-name)))
+           (let* ((script-name (url-decode (script-name*)))
                   (script-path (enough-url (regex-replace-all "\\\\" script-name "/")
                                            uri-prefix))
                   (script-path-directory (pathname-directory script-path)))
@@ -205,45 +208,37 @@ it'll be the content type used for all files in the folder."
                               (eq (first script-path-directory) :relative)
                               (loop for component in (rest script-path-directory)
                                     always (stringp component))))
-               (setf (return-code) +http-forbidden+)
-               (throw 'handler-done nil))
+               (setf (return-code*) +http-forbidden+)
+               (abort-request-handler))
              (handle-static-file (merge-pathnames script-path base-path) content-type))))
     (create-prefix-dispatcher uri-prefix #'handler)))
 
 (defun no-cache ()
   "Adds appropriate headers to completely prevent caching on most browsers."
-  (setf (header-out "Expires")
+  (setf (header-out :expires)
           "Mon, 26 Jul 1997 05:00:00 GMT"
-        (header-out "Cache-Control")
+        (header-out :cache-control)
           "no-store, no-cache, must-revalidate, post-check=0, pre-check=0"
-        (header-out "Pragma")
+        (header-out :pragma)
           "no-cache"
-        (header-out "Last-Modified")
+        (header-out :last-modified)
           (rfc-1123-date))
   (values))
 
-(defun ssl-p ()
-  "Whether the current connection to the client is secure."
-  (cond ((server-mod-lisp-p *server*) (ssl-session-id *request*))
-        (t #-:hunchentoot-no-ssl (server-ssl-certificate-file *server*)
-           #+:hunchentoot-no-ssl nil)))
-
 (defun redirect (target &key (host (host *request*) host-provided-p)
                              port
                              (protocol (if (ssl-p) :https :http))
                              (add-session-id (not (or host-provided-p
                                                       (starts-with-scheme-p target)
-                                                      (cookie-in *session-cookie-name*))))
-                             code)
+                                                      (cookie-in (session-cookie-name *acceptor*)))))
+                             (code +http-moved-temporarily+))
   "Redirects the browser to TARGET which should be a string.  If
 TARGET is a full URL starting with a scheme, HOST, PORT and PROTOCOL
 are ignored.  Otherwise, TARGET should denote the path part of a URL,
 PROTOCOL must be one of the keywords :HTTP or :HTTPS, and the URL to
 redirect to will be constructed from HOST, PORT, PROTOCOL, and TARGET.
 Adds a session ID if ADD-SESSION-ID is true.  If CODE is a 3xx
-redirection code, it will be sent as status code.  In case of NIL, a 
-302 status code will be sent to the client."
-  (setf code (or code +http-moved-temporarily+))
+redirection code, it will be sent as status code."
   (check-type code (integer 300 399))
   (let ((url (if (starts-with-scheme-p target)
                target
@@ -258,14 +253,14 @@ redirection code, it will be sent as status code.  In case of NIL, a
     (when add-session-id
       (setq url (add-cookie-value-to-url url :replace-ampersands-p nil)))
     (setf (header-out :location) url
-          (return-code *reply*) code)
-    (throw 'handler-done nil)))
+          (return-code*) code)
+    (abort-request-handler)))
 
 (defun require-authorization (&optional (realm "Hunchentoot"))
   "Sends back appropriate headers to require basic HTTP authentication
 \(see RFC 2617) for the realm REALM."
-  (setf (header-out "WWW-Authenticate")
+  (setf (header-out :www-authenticate)
           (format nil "Basic realm=\"~A\"" (quote-string realm))
         (return-code *reply*)
           +http-authorization-required+)
-  (throw 'handler-done nil))
+  (abort-request-handler))
