@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HUNCHENTOOT; Base: 10 -*-
 ;;; $Header: /usr/local/cvsrep/hunchentoot/request.lisp,v 1.35 2008/02/13 16:02:18 edi Exp $
 
-;;; Copyright (c) 2004-2008, Dr. Edmund Weitz.  All rights reserved.
+;;; Copyright (c) 2004-2009, Dr. Edmund Weitz.  All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -30,36 +30,52 @@
 (in-package :hunchentoot)
 
 (defclass request ()
-  ((headers-in :initarg :headers-in
-               :documentation "An alist of the incoming headers.  Note
-that these might be the headers coming in from mod_lisp which are
-different from the headers sent by the client.")
+  ((acceptor :initarg :acceptor
+             :documentation "The acceptor which created this request
+object."
+             :reader request-acceptor)
+   (headers-in :initarg :headers-in
+               :documentation "An alist of the incoming headers."
+               :reader headers-in)
    (method :initarg :method
-           :documentation "The request method as a keyword.  This slot
-is only filled if we're not behind mod_lisp.")
+           :documentation "The request method as a keyword."
+           :reader request-method)
    (uri :initarg :uri
-           :documentation "The request URI as a string.  This slot is
-only filled if we're not behind mod_lisp.")
+        :documentation "The request URI as a string."
+        :reader request-uri)
    (server-protocol :initarg :server-protocol
-                    :documentation "The HTTP protocol as a keyword.
-This slot is only filled if we're not behind mod_lisp.")
+                    :documentation "The HTTP protocol as a keyword."
+                    :reader server-protocol)
+   (remote-addr :initarg :remote-addr
+                :documentation "The IP address of the client that
+initiated this request."
+                :reader remote-addr)
+   (remote-port :initarg :remote-port
+                :documentation "The TCP port number of the client
+socket from which this request originated."
+                :reader remote-port)
    (content-stream :initarg :content-stream
                    :reader content-stream
                    :documentation "A stream from which the request
 body can be read if there is one.")
    (cookies-in :initform nil
-               :documentation "An alist of the cookies sent by the client.")
+               :documentation "An alist of the cookies sent by the client."
+               :reader cookies-in)
    (get-parameters :initform nil
                    :documentation "An alist of the GET parameters sent
-by the client.")
+by the client."
+                   :reader get-parameters)
    (post-parameters :initform nil
                     :documentation "An alist of the POST parameters
-sent by the client.")
+sent by the client."
+                    :reader post-parameters)
    (script-name :initform nil
                 :documentation "The URI requested by the client without
-the query string.")
+the query string."
+                :reader script-name)
    (query-string :initform nil
-                 :documentation "The query string of this request.")
+                 :documentation "The query string of this request."
+                 :reader query-string)
    (session :initform nil
             :accessor session
             :documentation "The session object associated with this
@@ -73,9 +89,37 @@ arbitrary data during the request.")
 POST request, populated only if not a multipart/form-data request."))
   (:documentation "Objects of this class hold all the information
 about an incoming request.  They are created automatically by
-Hunchentoot and can be accessed by the corresponding handler."))
+acceptors and can be accessed by the corresponding handler.
 
-(defun parse-rfc2388-form-data (stream content-type-header)
+You should not mess with the slots of these objects directly, but you
+can subclass REQUEST in order to implement your own behaviour.  See
+the REQUEST-CLASS slot of the ACCEPTOR class."))
+
+(defgeneric process-request (request)
+  (:documentation "This function is called by PROCESS-CONNECTION after
+the incoming headers have been read.  It selects and calls a handler
+and sends the output of this handler to the client using START-OUTPUT.
+It also sets up simple error handling for the request handler.  Note
+that PROCESS-CONNECTION is called once per connection and loops in
+case of a persistent connection while PROCESS-REQUEST is called anew
+for each request.
+
+Like PROCESS-CONNECTION, this might be a good place to introduce
+around methods which bind special variables or do other interesting
+things.
+
+The return value of this function is ignored."))
+
+(defun convert-hack (string external-format)
+  "The rfc2388 package is buggy in that it operates on a character
+stream and thus only accepts encodings which are 8 bit transparent.
+In order to support different encodings for parameter values
+submitted, we post process whatever string values the rfc2388 package
+has returned."
+  (flex:octets-to-string (map '(vector (unsigned-byte 8) *) 'char-code string)
+                         :external-format external-format))
+
+(defun parse-rfc2388-form-data (stream content-type-header external-format)
   "Creates an alist of POST parameters from the stream STREAM which is
 supposed to be of content type 'multipart/form-data'."
   (let* ((parsed-content-type-header (rfc2388:parse-header content-type-header :value))
@@ -96,7 +140,7 @@ supposed to be of content type 'multipart/form-data'."
                             (list contents
                                   (rfc2388:get-file-name headers)
                                   (rfc2388:content-type part :as-string t))
-                            contents))))))
+                            (convert-hack contents external-format)))))))
 
 (defun get-post-data (&key (request *request*) want-stream (already-read 0))
   "Reads the request body from the stream and stores the raw contents
@@ -105,22 +149,22 @@ object.  Returns just the stream if WANT-STREAM is true.  If there's a
 Content-Length header, it is assumed, that ALREADY-READ octets have
 already been read."
   (let* ((headers-in (headers-in request))
-         (content-length (when-let (content-length-header (cdr (assoc :content-length headers-in)))
+         (content-length (when-let (content-length-header (cdr (assoc :content-length headers-in
+                                                                      :test #'eq)))
                            (parse-integer content-length-header :junk-allowed t)))
          (content-stream (content-stream request)))
     (setf (slot-value request 'raw-post-data)
           (cond (want-stream
-                 (setf (flexi-stream-position *hunchentoot-stream*) 0)
-                 (when content-length
-                   (setf (flexi-stream-bound content-stream) content-length))
-                 content-stream)
+                 (let ((stream (make-flexi-stream content-stream :external-format +latin-1+)))
+                   (when content-length
+                     (setf (flexi-stream-bound stream) content-length))
+                   stream))
                 ((and content-length (> content-length already-read))
                  (decf content-length already-read)
                  (when (input-chunking-p)
                    ;; see RFC 2616, section 4.4
-                   (log-message :warn "Got Content-Length header although input chunking is on."))
+                   (log-message :warning "Got Content-Length header although input chunking is on."))
                  (let ((content (make-array content-length :element-type 'octet)))
-                   #+:clisp (setf (flexi-stream-element-type content-stream) 'octet)
                    (read-sequence content content-stream)
                    content))
                 ((input-chunking-p)
@@ -137,25 +181,10 @@ already been read."
   "The only initarg for a REQUEST object is :HEADERS-IN.  All other
 slot values are computed in this :AFTER method."
   (declare (ignore init-args))
-  (with-slots (headers-in cookies-in get-parameters post-parameters script-name query-string session)
+  (with-slots (headers-in cookies-in get-parameters script-name query-string session)
       request
     (handler-case
         (progn
-          (when (server-mod-lisp-p *server*)
-            ;; convert these two values to keywords
-            (let ((method-pair (assoc :method headers-in)))
-              (setf (cdr method-pair) (make-keyword (cdr method-pair))))
-            (let ((protocol-pair (assoc :server-protocol headers-in)))
-              (setf (cdr protocol-pair) (make-keyword (cdr protocol-pair))))
-            ;; and convert these two values to integers
-            (let ((remote-ip-port-pair (assoc :remote-ip-port headers-in)))
-              (setf (cdr remote-ip-port-pair) (parse-integer (cdr remote-ip-port-pair)
-                                                             :junk-allowed t)))
-            (let ((server-ip-port-pair (assoc :server-ip-port headers-in)))
-              (setf (cdr server-ip-port-pair) (parse-integer (cdr server-ip-port-pair)
-                                                             :junk-allowed t))))
-          ;; compute SCRIPT-NAME and QUERY-STRING slots from
-          ;; REQUEST_URI environment variable
           (let* ((uri (request-uri request))
                  (match-start (position #\? uri)))
             (cond
@@ -171,108 +200,179 @@ slot values are computed in this :AFTER method."
           (setq get-parameters
                 (form-url-encoded-list-to-alist (split "&" query-string))
                 cookies-in
-                (form-url-encoded-list-to-alist (split "\\s*[,;]\\s*" (cdr (assoc :cookie headers-in)))
+                (form-url-encoded-list-to-alist (split "\\s*[,;]\\s*" (cdr (assoc :cookie headers-in
+                                                                                  :test #'eq)))
                                                 +utf-8+)
                 session (session-verify request)
-                *session* session)
-          ;; if the content-type is 'application/x-www-form-urlencoded'
-          ;; or 'multipart/form-data', compute the post parameters from
-          ;; the content body
-          (when (member (request-method request) *methods-for-post-parameters* :test #'eq)
-            (when-let (content-type (cdr (assoc :content-type headers-in)))
-              (multiple-value-bind (type subtype external-format)
-                  (parse-content-type content-type t)
-                (setq post-parameters
-                      (cond ((and (string-equal type "application")
-                                  (string-equal subtype "x-www-form-urlencoded"))
-                             (unless (or (assoc :content-length headers-in)
-                                         (input-chunking-p))
-                               (error "Can't read request body because there's no~
-Content-Length header and input chunking is off."))
-                             (form-url-encoded-list-to-alist
-                              (split "&" (raw-post-data :request request
-                                                        ;; ASCII would suffice according to RFC...
-                                                        :external-format +latin-1+))
-                              external-format))
-                            ((and (string-equal type "multipart")
-                                  (string-equal subtype "form-data"))
-                             (setf (slot-value request 'raw-post-data) t)                           
-                             (handler-case
-                                 (let* ((*request* request)
-                                        (content-stream (content-stream request))
-                                        (start (flexi-stream-position content-stream)))
-                                   (prog1
-                                       (parse-rfc2388-form-data content-stream content-type)
-                                     (let* ((end (flexi-stream-position content-stream))
-                                            (stray-data (get-post-data :already-read (- end start))))
-                                       (when (and stray-data (plusp (length stray-data)))
-                                         (warn "~A octets of stray data after form-data sent by client."
-                                               (length stray-data))))
-                                     (setf (slot-value request 'raw-post-data) t)))
-                               (error (msg)
-                                 (log-message :error
-                                              "While parsing multipart/form-data parameters: ~A"
-                                              msg)
-                                 nil)))))))))
-      (error (cond)
-        (log-message* "Error when creating REQUEST object: ~A" cond)
+                *session* session))
+      (error (condition)
+        (log-message :error "Error when creating REQUEST object: ~A" condition)
         ;; we assume it's not our fault...
-        (setf (return-code) +http-bad-request+)))))
+        (setf (return-code*) +http-bad-request+)))))
+
+(defmethod process-request (request)
+  "Standard implementation for processing a request.  You should not
+change or replace this functionality unless you know what you're
+doing."
+  (let (*tmp-files* *headers-sent*)
+    (unwind-protect
+         (let* ((*request* request))
+           (multiple-value-bind (body error)
+               (catch 'handler-done
+                 (handler-bind ((error
+                                 (lambda (cond)
+                                   (when *log-lisp-errors-p*
+                                     (log-message *lisp-errors-log-level* "~A" cond))
+                                   ;; if the headers were already sent
+                                   ;; the error happens within the body
+                                   ;; and we have to close the stream
+                                   (when *headers-sent*
+                                     (setq *close-hunchentoot-stream* t))
+                                   (throw 'handler-done
+                                     (values nil cond))))
+                                (warning
+                                 (lambda (cond)
+                                   (when *log-lisp-warnings-p*
+                                     (log-message *lisp-warnings-log-level* "~A" cond)))))
+                   ;; skip dispatch if bad request
+                   (when (eql (return-code *reply*) +http-ok+)
+                     ;; now do the work
+                     (funcall (acceptor-request-dispatcher *acceptor*) *request*))))
+             (when error
+               (setf (return-code *reply*)
+                     +http-internal-server-error+))
+             (start-output :content (cond ((and error *show-lisp-errors-p*)
+                                           (format nil "<pre>~A</pre>"
+                                                   (escape-for-html (format nil "~A" error))))
+                                          (error
+                                           "An error has occured.")
+                                          (t body)))))
+      (dolist (path *tmp-files*)
+        (when (and (pathnamep path) (probe-file path))
+          ;; the handler may have chosen to (re)move the uploaded
+          ;; file, so ignore errors that happen during deletion
+          (ignore-errors
+            (delete-file path)))))))
+
+(defun parse-multipart-form-data (request external-format)
+  "Parse the REQUEST body as multipart/form-data, assuming that its
+content type has already been verified.  Returns the form data as
+alist or NIL if there was no data or the data could not be parsed."
+  (handler-case
+      (let ((content-stream (make-flexi-stream (content-stream request) :external-format +latin-1+)))
+        (prog1
+            (parse-rfc2388-form-data content-stream (header-in :content-type request) external-format)
+          (let ((stray-data (get-post-data :already-read (flexi-stream-position content-stream))))
+            (when (and stray-data (plusp (length stray-data)))
+              (hunchentoot-warn "~A octets of stray data after form-data sent by client."
+                                (length stray-data))))))
+    (error (condition)
+      (log-message :error "While parsing multipart/form-data parameters: ~A" condition)
+      nil)))
+
+(defun maybe-read-post-parameters (&key (request *request*) force external-format)
+  "Make surce that any POST parameters in the REQUEST are parsed.  The
+body of the request must be either application/x-www-form-urlencoded
+or multipart/form-data to be considered as containing POST parameters.
+If FORCE is true, parsing is done unconditionally.  Otherwise, parsing
+will only be done if the RAW-POST-DATA slot in the REQUEST is false.
+EXTERNAL-FORMAT specifies the external format of the data in the
+request body.  By default, the encoding is determined from the
+Content-Type header of the request or from
+*HUNCHENTOOT-DEFAULT-EXTERNAL-FORMAT* if none is found."
+  (when (and (header-in :content-type request)
+             (member (request-method request) *methods-for-post-parameters* :test #'eq)
+             (or force
+                 (not (slot-value request 'raw-post-data))))
+    (unless (or (header-in :content-length request)
+                (input-chunking-p))
+      (log-message :warning "Can't read request body because there's ~
+no Content-Length header and input chunking is off.")
+      (return-from maybe-read-post-parameters nil))
+    (handler-case
+        (multiple-value-bind (type subtype charset)
+              (parse-content-type (header-in :content-type request))
+          (let ((external-format (or external-format
+                                     (when charset
+                                       (handler-case
+                                           (make-external-format charset :eol-style :lf)
+                                         (error ()
+                                           (hunchentoot-warn "Ignoring ~
+unknown character set ~A in request content type."
+                                                 charset))))
+                                     *hunchentoot-default-external-format*)))
+            (setf (slot-value request 'post-parameters)
+                  (cond ((and (string-equal type "application")
+                              (string-equal subtype "x-www-form-urlencoded"))
+                         (form-url-encoded-list-to-alist
+                          (split "&" (raw-post-data :external-format +latin-1+))
+                          external-format))
+                        ((and (string-equal type "multipart")
+                              (string-equal subtype "form-data"))
+                         (prog1 (parse-multipart-form-data request external-format)
+                           (setf (slot-value request 'raw-post-data) t)))))))
+      (error (condition)
+        (log-message :error "Error when reading POST parameters from body: ~A" condition)
+        ;; this is not the right thing to do because it could happen
+        ;; that we aren't finished reading from the request stream and
+        ;; can't send a reply - to be revisited
+        (setf (return-code*) +http-bad-request+
+              *close-hunchentoot-stream* t)
+        (abort-request-handler)))))
 
 (defun recompute-request-parameters (&key (request *request*)
                                           (external-format *hunchentoot-default-external-format*))
   "Recomputes the GET and POST parameters for the REQUEST object
 REQUEST.  This only makes sense if you're switching external formats
 during the request."
-  (with-slots (headers-in get-parameters post-parameters query-string)
-      request
-    (setq get-parameters
-          (form-url-encoded-list-to-alist (split "&" query-string) external-format)
-          post-parameters
-          (when-let (raw-post-data (raw-post-data :request request
-                                                  :external-format +latin-1+))
-            (and (when-let (content-type (cdr (assoc :content-type headers-in)))
-                   (multiple-value-bind (type subtype)
-                       (parse-content-type content-type)
-                     (and (string-equal type "application")
-                          (string-equal subtype "x-www-form-urlencoded"))))
-                 (form-url-encoded-list-to-alist (split "&" raw-post-data) external-format)))))
+  (maybe-read-post-parameters :request request :force t :external-format external-format)
+  (setf (slot-value request 'get-parameters)
+        (form-url-encoded-list-to-alist (split "&" (query-string request)) external-format))
   (values))
                                                 
-(defun script-name (&optional (request *request*))
+(defun script-name* (&optional (request *request*))
   "Returns the file name of the REQUEST object REQUEST. That's the
 requested URI without the query string \(i.e the GET parameters)."
-  (slot-value request 'script-name))
+  (script-name request))
 
-(defun query-string (&optional (request *request*))
+(defun query-string* (&optional (request *request*))
   "Returns the query string of the REQUEST object REQUEST. That's
 the part behind the question mark \(i.e. the GET parameters)."
-  (slot-value request 'query-string))
+  (query-string request))
 
-(defun get-parameters (&optional (request *request*))
+(defun get-parameters* (&optional (request *request*))
   "Returns an alist of the GET parameters associated with the REQUEST
 object REQUEST."
-  (slot-value request 'get-parameters))
+  (get-parameters request))
 
-(defun post-parameters (&optional (request *request*))
+(defmethod post-parameters :before ((request request))
+  (maybe-read-post-parameters :request request))
+
+(defun post-parameters* (&optional (request *request*))
   "Returns an alist of the POST parameters associated with the REQUEST
 object REQUEST."
-  (slot-value request 'post-parameters))
+  (post-parameters request))
 
-(defun headers-in (&optional (request *request*))
+(defun headers-in* (&optional (request *request*))
   "Returns an alist of the incoming headers associated with the
 REQUEST object REQUEST."
-  (slot-value request 'headers-in))
+  (headers-in request))
 
-(defun cookies-in (&optional (request *request*))
+(defun cookies-in* (&optional (request *request*))
   "Returns an alist of all cookies associated with the REQUEST object
 REQUEST."
-  (slot-value request 'cookies-in))
+  (cookies-in request))
 
-(defun header-in (name &optional (request *request*))
+(defgeneric header-in (name request)
+  (:documentation "Returns the incoming header with name NAME.  NAME
+can be a keyword \(recommended) or a string.")
+  (:method (name request)
+   (cdr (assoc* name (headers-in request)))))
+
+(defun header-in* (name &optional (request *request*))
   "Returns the incoming header with name NAME.  NAME can be a keyword
 \(recommended) or a string."
-  (cdr (assoc name (headers-in request))))
+  (header-in name request))
 
 (defun authorization (&optional (request *request*))
   "Returns as two values the user and password \(if any) as encoded in
@@ -287,10 +387,13 @@ the 'AUTHORIZATION' header.  Returns NIL if there is no such header."
           (split ":" (base64:base64-string-to-string (subseq authorization start)))
         (values user password)))))
 
-(defun remote-addr (&optional (request *request*))
+(defun remote-addr* (&optional (request *request*))
   "Returns the address the current request originated from."
-  (cond ((server-mod-lisp-p *server*) (header-in :remote-ip-addr request))
-        (t *remote-host*)))
+  (remote-addr request))
+
+(defun remote-port* (&optional (request *request*))
+  "Returns the port the current request originated from."
+  (remote-port request))
 
 (defun real-remote-addr (&optional (request *request*))
   "Returns the 'X-Forwarded-For' incoming http header as the
@@ -302,56 +405,21 @@ Otherwise returns the value of REMOTE-ADDR as the only value."
                              (values (first addresses) addresses)))
           (t (remote-addr request)))))
 
-(defun server-addr (&optional (request *request*))
-  "Returns the address at which the current request arrived."
-  (cond ((server-mod-lisp-p *server*) (header-in :server-ip-addr request))
-        (t *local-host*)))
-
-(defun remote-port (&optional (request *request*))
-  "Returns the port the current request originated from."
-  (cond ((server-mod-lisp-p *server*) (header-in :remote-ip-port request))
-        (t *remote-port*)))
-
-(defun server-port (&optional (request *request*))
-  "Returns the port at which the current request arrived."
-  (cond ((server-mod-lisp-p *server*) (header-in :server-ip-port request))
-        (t (server-local-port *server*))))
-
 (defun host (&optional (request *request*))
   "Returns the 'Host' incoming http header value."
   (header-in :host request))
 
-(defun request-uri (&optional (request *request*))
+(defun request-uri* (&optional (request *request*))
   "Returns the request URI."
-  (cond ((server-mod-lisp-p *server*) (header-in :url request))
-        (t (slot-value request 'uri))))
+  (request-uri request))
 
-(defun request-method (&optional (request *request*))
+(defun request-method* (&optional (request *request*))
   "Returns the request method as a Lisp keyword."
-  (cond ((server-mod-lisp-p *server*) (header-in :method request))
-        (t (slot-value request 'method))))
+  (request-method request))
 
-(defun server-protocol (&optional (request *request*))
+(defun server-protocol* (&optional (request *request*))
   "Returns the request protocol as a Lisp keyword."
-  (cond ((server-mod-lisp-p *server*) (header-in :server-protocol request))
-        (t (slot-value request 'server-protocol))))
-
-(defun mod-lisp-id (&optional (request *request*))
-  "Returns the 'Server ID' sent by mod_lisp.  This value is set in
-Apache's server configuration file and is of course only available if
-mod_lisp is the front-end."
-  (and (or (server-mod-lisp-p *server*)
-           (warn "Calling MOD-LISP-ID although ~S is a stand-alone server."
-                 *server*))
-       (header-in :server-id request)))
-
-(defun ssl-session-id (&optional (request *request*))
-  "Returns the 'SSL_SESSION_ID' header sent my mod_lisp and is of
-course only available if mod_lisp is the front-end."
-  (and (or (server-mod-lisp-p *server*)           
-           (warn "Calling SSL-SESSION-ID although ~S is a stand-alone server."
-                 *server*))
-       (header-in :ssl-session-id request)))
+  (server-protocol request))
 
 (defun user-agent (&optional (request *request*))
   "Returns the 'User-Agent' http header."
@@ -393,11 +461,24 @@ TIME."
     ;; simple string comparison is sufficient; see RFC 2616 14.25
     (when (and if-modified-since
                (equal if-modified-since time-string))
-      (setf (return-code) +http-not-modified+)
-      (throw 'handler-done nil))
+      (setf (return-code*) +http-not-modified+)
+      (abort-request-handler))
     (values)))
 
-(defun raw-post-data (&key (request *request*) external-format force-binary force-text want-stream)
+(defun external-format-from-content-type (content-type)
+  "Creates and returns an external format corresponding to the value
+of the content type header provided in CONTENT-TYPE.  If the content
+type was not set or if the character set specified was invalid, NIL is
+returned."
+  (when content-type
+    (when-let (charset (nth-value 2 (parse-content-type content-type)))
+      (handler-case
+          (make-external-format (as-keyword charset) :eol-style :lf)
+        (error ()
+          (hunchentoot-warn "Invalid character set ~S in request has been ignored."
+                            charset))))))
+
+(defun raw-post-data (&key (request *request*) external-format force-text force-binary want-stream)
   "Returns the content sent by the client if there was any \(unless
 the content type was \"multipart/form-data\").  By default, the result
 is a string if the type of the `Content-Type' media type is \"text\",
@@ -428,11 +509,11 @@ mix calls which have different values for WANT-STREAM.
 Note that this function is slightly misnamed because a client can send
 content even if the request method is not POST."
   (when (and force-binary force-text)
-    (error "It doesn't make sense to set both FORCE-BINARY and FORCE-TEXT to a true value."))
+    (parameter-error "It doesn't make sense to set both FORCE-BINARY and FORCE-TEXT to a true value."))
   (unless (or external-format force-binary)
-    (setq external-format
-          (when-let (content-type (cdr (assoc :content-type (headers-in request))))
-            (nth-value 2 (parse-content-type content-type force-text)))))
+    (setq external-format (or (external-format-from-content-type (header-in :content-type request))
+                              (when force-text
+                                *hunchentoot-default-external-format*))))
   (let ((raw-post-data (or (slot-value request 'raw-post-data)
                            (get-post-data :request request :want-stream want-stream))))
     (cond ((typep raw-post-data 'stream) raw-post-data)
@@ -445,7 +526,7 @@ content even if the request method is not POST."
 REQUEST \(the default is the current request) if it exists.  The
 second return value is true if such a value was found."
   (when request
-    (let ((found (assoc symbol (aux-data request))))
+    (let ((found (assoc symbol (aux-data request) :test #'eq)))
       (values (cdr found) found))))
 
 (defsetf aux-request-value (symbol &optional request)
@@ -456,7 +537,7 @@ associated with SYMBOL it will be replaced."
   (with-rebinding (symbol)
     (with-unique-names (place %request)
       `(let* ((,%request (or ,request *request*))
-              (,place (assoc ,symbol (aux-data ,%request))))
+              (,place (assoc ,symbol (aux-data ,%request) :test #'eq)))
          (cond
            (,place
             (setf (cdr ,place) ,new-value))
