@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HUNCHENTOOT; Base: 10 -*-
 ;;; $Header: /usr/local/cvsrep/hunchentoot/specials.lisp,v 1.33 2008/04/08 14:39:18 edi Exp $
 
-;;; Copyright (c) 2004-2008, Dr. Edmund Weitz. All rights reserved.
+;;; Copyright (c) 2004-2009, Dr. Edmund Weitz. All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -29,6 +29,11 @@
 
 (in-package :hunchentoot)
 
+(defmacro defconstant (name value &optional doc)
+  "Make sure VALUE is evaluated only once \(to appease SBCL)."
+  `(cl:defconstant ,name (if (boundp ',name) (symbol-value ',name) ,value)
+     ,@(when doc (list doc))))
+
 (eval-when (:compile-toplevel :execute :load-toplevel)
   (defmacro defvar-unbound (name &optional (doc-string ""))
     "Convenience macro to declare unbound special variables with a
@@ -50,8 +55,11 @@ server's status line."
                                           value reason-phrase))
        (setf (gethash ,value *http-reason-phrase-map*) ,reason-phrase))))
 
-(defconstant +crlf+ #.(format nil "~C~C" #\Return #\Linefeed)
-  "A constant string consisting of the two ASCII characters CR and LF.")
+(defconstant +crlf+
+  (make-array 2 :element-type '(unsigned-byte 8)
+              :initial-contents (mapcar 'char-code '(#\Return #\Linefeed)))
+  "A 2-element array consisting of the character codes for a CRLF
+sequence.")
 
 (def-http-return-code +http-continue+ 100 "Continue")
 (def-http-return-code +http-switching-protocols+ 101 "Switching Protocols")
@@ -112,10 +120,6 @@ for cookie date format.")
   "The three-character names of the twelve months - needed for cookie
 date format.")
 
-(defvar *session-cookie-name* "hunchentoot-session"
-  "The name of the cookie \(or the GET parameter) which is used to
-store the session on the client side.")
-
 (defvar *rewrite-for-session-urls* t
   "Whether HTML pages should possibly be rewritten for cookie-less
 session-management.")
@@ -129,7 +133,11 @@ session-management.")
   "A fresh random state.")
 
 (defvar-unbound *session-secret*
-  "A random value that's used to encode the public session data.")
+  "A random ASCII string that's used to encode the public session
+data.  This variable is initially unbound and will be set \(using
+RESET-SESSION-SECRET) the first time a session is created, if
+necessary.  You can prevent this from happening if you set the value
+yourself before starting acceptors.")
 
 (defvar-unbound *hunchentoot-stream*
   "The stream representing the socket Hunchentoot is listening on.")
@@ -149,18 +157,16 @@ Hunchentoot.  The pathname denotes the temporary file to which
 the uploaded file is written.  The hook is called directly before
 the file is created.")
 
-(defvar *session-data* nil
-  "All sessions of all users currently using Hunchentoot.  An
-alist where the car is the session's ID and the cdr is the
-SESSION object itself.")
+(defvar *session-db* nil
+  "The default \(global) session database.")
 
 (defvar *session-max-time* #.(* 30 60)
   "The default time \(in seconds) after which a session times out.")
 
 (defvar *session-gc-frequency* 50
   "A session GC \(see function SESSION-GC) will happen every
-*SESSION-GC-FREQUENCY* requests \(counting only requests which
-use a session) if this variable is not NIL.")
+*SESSION-GC-FREQUENCY* requests \(counting only requests which create
+a new session) if this variable is not NIL.  See SESSION-CREATED.")
 
 (defvar *use-user-agent-for-sessions* t
   "Whether the 'User-Agent' header should be encoded into the session
@@ -188,66 +194,60 @@ which incoming and outgoing headers will be written for debugging
 purposes.")
 
 (defvar *show-lisp-errors-p* nil
-  "Whether Lisp errors should be shown in HTML output.")
-
-(defvar *show-lisp-backtraces-p* nil
-  "Whether Lisp backtraces should be shown in HTML output when an
-error occurs.  Will only have an effect if *SHOW-LISP-ERRORS-P* is
-also true.")
+  "Whether Lisp errors in request handlers should be shown in HTML output.")
 
 (defvar *log-lisp-errors-p* t
-  "Whether Lisp errors should be logged.")
+  "Whether Lisp errors in request handlers should be logged.")
 
 (defvar *log-lisp-warnings-p* t
-  "Whether Lisp warnings should be logged.")
-
-(defvar *log-lisp-backtraces-p* nil
-  "Whether Lisp backtraces should be logged when an error or warning
-occurs.  Will only have an effect if *LOG-LISP-ERRORS-P* or
-*LOG-LISP-BACKTRACES* are also true.")
+  "Whether Lisp warnings in request handlers should be logged.")
 
 (defvar *lisp-errors-log-level* :error
-  "Log level for Lisp errors.")
+  "Log level for Lisp errors.  Should be one of :ERROR \(the default),
+:WARNING, or :INFO.")
 
 (defvar *lisp-warnings-log-level* :warning
-  "Log level for Lisp warnings.")
+  "Log level for Lisp warnings.  Should be one of :ERROR, :WARNING
+\(the default), or :INFO.")
 
-(defvar *show-access-log-messages* t
-  "Whether routine messages about each request should be logged.  This
-will only be done if SERVER-USE-APACHE-LOG-P is NIL.")
+(defvar *message-log-pathname* nil
+  "A designator for the pathname of the message log file used by the
+LOG-MESSAGE-TO-FILE function.  The initial value is NIL which means
+that nothing will be logged.")
 
-(defvar *log-file*
-    (load-time-value
-      (let ((tmp-dir
-              #+:allegro (system:temporary-directory)
-              #+:lispworks (pathname (or (lw:environment-variable "TEMP")
-                                         (lw:environment-variable "TMP")
-                                         #+:win32 "C:/"
-                                         #-:win32 "/tmp/"))
-              #-(or :allegro :lispworks) #p"/tmp/"))
-        (merge-pathnames "hunchentoot.log" tmp-dir)))
-  "The log file to use \(unless the Apache log is used).")
+(defvar *access-log-pathname* nil
+  "A designator for the pathname of the access log file used by the
+LOG-ACCESS-TO-FILE function.  The initial value is NIL which means
+that nothing will be logged.")
 
-(defvar *log-file-stream* nil
-  "The stream corresponding to the log file.")
+(defvar *message-log-lock* (make-lock "global-message-log-lock")
+  "A global lock to prevent concurrent access to the log file
+used by LOG-MESSAGE-TO-FILE function.")
 
-(defvar *log-file-lock* (make-lock "log-file-lock")
-  "A lock to prevent two threads from writing to the log file at
-same time.")
-  
-(defvar-unbound *session*
-  "The current SESSION object.")
+(defvar *access-log-lock* (make-lock "global-access-log-lock")
+  "A global lock to prevent concurrent access to the log file
+used by LOG-ACCESS-TO-FILE function.")
+
+(defvar-unbound *acceptor*
+  "The current ACCEPTOR object while in the context of a request.")
 
 (defvar-unbound *request*
-  "The current REQUEST object.")
+  "The current REQUEST object while in the context of a request.")
 
 (defvar-unbound *reply*
-  "The current REPLY object.")
+  "The current REPLY object while in the context of a request.")
 
-(defvar *log-prefix* t
-  "The prefix which is printed in front of Apache log
-messages. This should be a string or T \(for \"Hunchentoot\", the
-default) or NIL \(meaning no prefix).")
+(defvar-unbound *session*
+  "The current session while in the context of a request, or NIL.")
+
+(defvar *break-even-while-reading-request-type-p* nil
+  "If this variable is set to true, Hunchentoot will not bind
+*BREAK-ON-SIGNALS* to NIL while reading the next request type from an
+incoming connection.  By default, Hunchentoot will not enter the
+debugger if an error occurs during the reading of the request type, as
+this will happen regularily and legitimately.  \(The incoming
+connection times out or the client closes the connection without
+initiating another request, which is permissible.)")
 
 (defconstant +implementation-link+
   #+:cmu "http://www.cons.org/cmucl/"
@@ -257,8 +257,8 @@ default) or NIL \(meaning no prefix).")
   #+:openmcl "http://openmcl.clozure.com/"
   "A link to the website of the underlying Lisp implementation.")
 
-(defvar *dispatch-table* (list 'default-dispatcher)
-  "A list of dispatch functions - see *META-DISPATCHER*.")
+(defvar *dispatch-table* (list 'dispatch-easy-handlers 'default-dispatcher)
+  "A global list of dispatch functions.")
 
 (defvar *default-handler* 'default-handler
   "The name of the function which is always returned by
@@ -275,18 +275,11 @@ called if the content handler has set a return code which is not in
 
 (defvar *handle-http-errors-p* t
   "A generalized boolean that determines whether return codes which
-are not in *APPROVED-HEADERS* are treated specially.  When its value
-is true \(the default), either a default body for the return code or
-the result of calling *HTTP-ERROR-HANDLER* is used.  When the value is
-NIL, no special action is taken and you are expected to supply your
-own response body to describe the error.")
-
-(defvar *default-log-level* nil
-  "The default log level for LOG-MESSAGE*.")
-
-(defvar *session-data-lock* (make-lock "session-data-lock")
-  "A lock to prevent two threads from modifying *SESSION-DATA* at the
-same time.")
+are not in *APPROVED-RETURN-CODES* are treated specially.  When its
+value is true \(the default), either a default body for the return
+code or the result of calling *HTTP-ERROR-HANDLER* is used.  When the
+value is NIL, no special action is taken and you are expected to
+supply your own response body to describe the error.")
 
 (defvar *session-removal-hook* (constantly nil)
   "A function of one argument \(a session object) which is called
@@ -316,54 +309,22 @@ encode cookie values.")
 (defconstant +buffer-length+ 8192
   "Length of buffers used for internal purposes.")
 
-(defvar-unbound *server*
-  "During the execution of dispatchers and handlers this variable
-is bound to the SERVER object which processes the request.")
+(defvar *default-connection-timeout* 20
+  "The default connection timeout used when an acceptor is reading
+from and writing to a socket stream.")
 
-(defvar *meta-dispatcher* (lambda (server)
-                            (declare (ignore server))
-                            *dispatch-table*)
-  "The value of this variable should be a function of one argument.
-It is called with the current Hunchentoot server instance \(unless the
-server has its own dispatch table) and must return a suitable dispatch
-table.  The initial value is a function which always unconditionally
-returns *DISPATCH-TABLE*.")
+(define-symbol-macro *supports-threads-p*
+  #+:lispworks t
+  #-:lispworks bt:*supports-threads-p*)
 
-(defvar *server-counter* 0
-  "Internal counter used to generate meaningful names for
-listener threads.")
+(defvar *global-session-db-lock*
+  (load-time-value (and *supports-threads-p* (make-lock "global-session-db-lock")))
+  "A global lock to prevent two threads from modifying *session-db* at
+the same time \(or NIL for Lisps which don't have threads).")
 
-(defvar *worker-counter* 0
-  "Internal counter used to generate meaningful names for worker
-threads.")
-
-(defvar *default-read-timeout* 20
-  "The default read-timeout used when a Hunchentoot server is
-reading from a socket stream.")
-
-(defvar *default-write-timeout* 20
-  "The default write-timeout used when a Hunchentoot server is
-writing to a socket stream.")
-
-(defvar *cleanup-interval* 100
-  "Should be NIL or a positive integer.  The system calls
-*CLEANUP-FUNCTION* whenever *CLEANUP-INTERVAL* new worker threads have
-been created unless the value is NIL.")
-
-(defvar *cleanup-function* 'cleanup-function
-  "The function which is called if *CLEANUP-INTERVAL* is not NIL.")
-
-(defvar-unbound *local-host*
-  "Bound to a string denoting the address at which the current
-request arrived.")
-
-(defvar-unbound *remote-host*
-  "Bound to a string denoting the address the current request
-originated from.")
-
-(defvar-unbound *remote-port*
-  "Bound to an integer denoting the port the current request
-originated from.")
+(defconstant +new-connection-wait-time+ 2
+  "Time in seconds to wait for a new connection to arrive before
+performing a cleanup run.")
 
 (pushnew :hunchentoot *features*)
 
