@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HUNCHENTOOT; Base: 10 -*-
 ;;; $Header: /usr/local/cvsrep/hunchentoot/util.lisp,v 1.35 2008/04/08 14:39:18 edi Exp $
 
-;;; Copyright (c) 2004-2008, Dr. Edmund Weitz. All rights reserved.
+;;; Copyright (c) 2004-2009, Dr. Edmund Weitz. All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -29,76 +29,6 @@
 
 (in-package :hunchentoot)
 
-#-:lispworks
-(defmacro when-let ((var form) &body body)
-  "Evaluates FORM and binds VAR to the result, then executes BODY
-if VAR has a true value."
-  `(let ((,var ,form))
-     (when ,var ,@body)))
-
-#-:lispworks
-(defmacro with-unique-names ((&rest bindings) &body body)
-  "Syntax: WITH-UNIQUE-NAMES ( { var | (var x) }* ) declaration* form*
-
-Executes a series of forms with each VAR bound to a fresh,
-uninterned symbol. The uninterned symbol is as if returned by a call
-to GENSYM with the string denoted by X - or, if X is not supplied, the
-string denoted by VAR - as argument.
-
-The variable bindings created are lexical unless special declarations
-are specified. The scopes of the name bindings and declarations do not
-include the Xs.
-
-The forms are evaluated in order, and the values of all but the last
-are discarded \(that is, the body is an implicit PROGN)."
-  ;; reference implementation posted to comp.lang.lisp as
-  ;; <cy3bshuf30f.fsf@ljosa.com> by Vebjorn Ljosa - see also
-  ;; <http://www.cliki.net/Common%20Lisp%20Utilities>
-  `(let ,(mapcar #'(lambda (binding)
-                     (check-type binding (or cons symbol))
-                     (if (consp binding)
-                       (destructuring-bind (var x) binding
-                         (check-type var symbol)
-                         `(,var (gensym ,(etypecase x
-                                          (symbol (symbol-name x))
-                                          (character (string x))
-                                          (string x)))))
-                       `(,binding (gensym ,(symbol-name binding)))))
-                 bindings)
-         ,@body))
-
-#+:lispworks
-(defmacro with-rebinding (bindings &body body)
-  "Renaming LW:REBINDING for better indentation."
-  `(lw:rebinding ,bindings ,@body))
-
-#-:lispworks
-(defmacro with-rebinding (bindings &body body)
-  "Syntax: WITH-REBINDING ( { var | (var prefix) }* ) form*
-
-Evaluates a series of forms in the lexical environment that is
-formed by adding the binding of each VAR to a fresh, uninterned
-symbol, and the binding of that fresh, uninterned symbol to VAR's
-original value, i.e., its value in the current lexical environment.
-
-The uninterned symbol is created as if by a call to GENSYM with the
-string denoted by PREFIX - or, if PREFIX is not supplied, the string
-denoted by VAR - as argument.
-
-The forms are evaluated in order, and the values of all but the last
-are discarded \(that is, the body is an implicit PROGN)."
-  ;; reference implementation posted to comp.lang.lisp as
-  ;; <cy3wv0fya0p.fsf@ljosa.com> by Vebjorn Ljosa - see also
-  ;; <http://www.cliki.net/Common%20Lisp%20Utilities>
-  (loop for binding in bindings
-        for var = (if (consp binding) (car binding) binding)
-        for name = (gensym)
-        collect `(,name ,var) into renames
-        collect ``(,,var ,,name) into temps
-        finally (return `(let ,renames
-                          (with-unique-names ,bindings
-                            `(let (,,@temps)
-                              ,,@body))))))
 
 (defun starts-with-p (seq subseq &key (test 'eql))
   "Tests whether the sequence SEQ starts with the sequence
@@ -125,47 +55,26 @@ digits."
       (format s "~VR" base
               (random base *the-random-state*)))))
 
-(defun reset-session-secret ()
-  "Sets *SESSION-SECRET* to a new random value. All old sessions will
-cease to be valid."
-  (setq *session-secret* (create-random-string 10 36)))
-
 (defun reason-phrase (return-code)
   "Returns a reason phrase for the HTTP return code RETURN-CODE
 \(which should be an integer) or NIL for return codes Hunchentoot
 doesn't know."
   (gethash return-code *http-reason-phrase-map*))
 
-(defun make-keyword (string &key (destructivep t))
-  "Interns the upcased version of STRING into the KEYWORD package.
-Uses NSTRING-UPCASE if DESTRUCTIVEP is true.  Returns NIL if STRING is
-not a string."
-  (and (stringp string)
-       (intern (if destructivep
-                 (nstring-upcase string)
-                 (string-upcase string)) :keyword)))
-
-(defgeneric assoc (thing alist &key &allow-other-keys)
-  (:documentation "LIKE CL:ASSOC, but \'does the right thing\' if
-THING is a string or a symbol."))
-
-(defmethod assoc ((thing symbol) alist &key &allow-other-keys)
-  "Version of ASSOC for symbols, always uses EQ as test function."
-  (cl:assoc thing alist :test #'eq))
-
-(defmethod assoc ((thing string) alist &key (test #'string-equal))
-  "Version of ASSOC for strings, uses STRING-EQUAL as default test
-function."
-  (cl:assoc thing alist :test test))
-
-(defmethod assoc (thing alist &key (test #'eql))
-  "Default method - uses EQL as default test like CL:ASSOC."
-  (cl:assoc thing alist :test test))
+(defgeneric assoc* (thing alist)
+  (:documentation "Similar to CL:ASSOC, but 'does the right thing' if
+THING is a string or a symbol.")
+  (:method ((thing symbol) alist)
+   (assoc thing alist :test #'eq))
+  (:method ((thing string) alist)
+   (assoc thing alist :test #'string-equal))
+  (:method (thing alist)
+   (assoc thing alist :test #'eql)))
 
 (defun md5-hex (string)
   "Calculates the md5 sum of the string STRING and returns it as a hex string."
   (with-output-to-string (s)
-    (loop for code across (md5:md5sum-sequence string)
+    (loop for code across (md5:md5sum-sequence (coerce string 'simple-string))
 	  do (format s "~2,'0x" code))))
 
 (defun escape-for-html (string)
@@ -250,32 +159,69 @@ called from the RFC2388 library when a file is uploaded."
                    ((#\") (write-string "\\\"" out))
                    (otherwise (write-char char out)))))))
 
+(defmacro upgrade-vector (vector new-type &key converter)
+  "Returns a vector with the same length and the same elements as
+VECTOR \(a variable holding a vector) but having element type
+NEW-TYPE.  If CONVERTER is not NIL, it should designate a function
+which will be applied to each element of VECTOR before the result is
+stored in the new vector.  The resulting vector will have a fill
+pointer set to its end.
+
+The macro also uses SETQ to store the new vector in VECTOR."
+  `(setq ,vector
+         (loop with length = (length ,vector)
+               with new-vector = (make-array length
+                                             :element-type ,new-type
+                                             :fill-pointer length)
+               for i below length
+               do (setf (aref new-vector i) ,(if converter
+                                               `(funcall ,converter (aref ,vector i))
+                                               `(aref ,vector i)))
+               finally (return new-vector))))
+
 (defun url-decode (string &optional (external-format *hunchentoot-default-external-format*))
   "Decodes a URL-encoded STRING which is assumed to be encoded using
 the external format EXTERNAL-FORMAT."
-  (let ((vector (make-array (length string)
-                            :element-type 'octet
-                            :fill-pointer 0)))
-    (loop with percent-p and buff
-          for char of-type character across string
-          for i from 0
-          when buff do
-            (vector-push (parse-integer string
-                                        :start (1- i)
-                                        :end (1+ i)
-                                        :radix 16)
-                         vector)
-            (setq buff nil)
-          else when percent-p
-                 do (setq buff t
-                          percent-p nil)
-          else when (char= char #\%)
-                 do (setq percent-p t)
-          else do (vector-push (char-code (case char
-                                            ((#\+) #\Space)
-                                            (otherwise char)))
-                         vector))
-    (octets-to-string vector :external-format external-format)))
+  (when (zerop (length string))
+    (return-from url-decode ""))
+  (let ((vector (make-array (length string) :element-type 'octet :fill-pointer 0))
+        (i 0)
+        unicodep)
+    (loop
+      (unless (< i (length string))
+        (return))
+      (let ((char (aref string i)))
+       (labels ((decode-hex (length)
+                  (prog1
+                      (parse-integer string :start i :end (+ i length) :radix 16)
+                    (incf i length)))
+                (push-integer (integer)
+                  (vector-push integer vector))
+                (peek ()
+                  (aref string i))
+                (advance ()
+                  (setq char (peek))
+                  (incf i)))
+         (cond
+          ((char= #\% char)
+           (advance)
+           (cond
+            ((char= #\u (peek))
+             (unless unicodep
+               (setq unicodep t)
+               (upgrade-vector vector '(integer 0 65535)))
+             (advance)
+             (push-integer (decode-hex 4)))
+            (t
+             (push-integer (decode-hex 2)))))
+          (t
+           (push-integer (char-code (case char
+                                      ((#\+) #\Space)
+                                      (otherwise char))))
+           (advance))))))
+    (cond (unicodep
+           (upgrade-vector vector 'character :converter #'code-char))
+          (t (octets-to-string vector :external-format external-format)))))
 
 (defun form-url-encoded-list-to-alist (form-url-encoded-list
                                        &optional (external-format *hunchentoot-default-external-format*))
@@ -305,94 +251,72 @@ alist.  Both names and values are url-decoded while doing this."
                                                                :external-format external-format)
                             do (format s "%~2,'0x" octet)))))))
 
-(defun parse-content-type (content-type-header &optional want-external-format-p)
+(defun parse-content-type (content-type-header)
   "Reads and parses a `Content-Type' header and returns it as three
-values - the type, the subtype, and an external format corresponding
-to the 'charset' parameter in the header \(or
-*HUNCHENTOOT-DEFAULT-EXTERNAL-FORMAT*), if there is one and if the
-content type is \"text\" or WANT-EXTERNAL-FORMAT-P is true.
-CONTENT-TYPE-HEADER is supposed to be the corresponding header value
-as a string."
-  (with-input-from-string (stream content-type-header)
-    (let* ((*current-error-message* "Corrupted Content-Type header:")
-           (type (read-token stream))
-           (subtype (and (or (ignore-errors (assert-char stream #\/))
-                             (return-from parse-content-type
-                               ;; try to return something meaningful
-                               (values "application" "octet-stream"
-                                       (and want-external-format-p
-                                            *hunchentoot-default-external-format*))))
-                         (read-token stream)))
-           (parameters (read-name-value-pairs stream))
-           (charset (cdr (assoc "charset" parameters)))
-           (external-format
-            (and (or want-external-format-p
-                     (string-equal type "text"))
-                 (or (when charset
-                       (handler-case
-                           (make-external-format (make-keyword charset) :eol-style :lf)
-                         (error (condition)
-                           (warn "Ignoring external format of name ~S~
-because of error:~%~A"
-                                 charset condition))))
-                     *hunchentoot-default-external-format*))))
-      (values type subtype external-format))))
+values - the type, the subtype, and the requests' character set as
+specified in the 'charset' parameter in the header, if there is one
+and if the content type is \"text\".  CONTENT-TYPE-HEADER is supposed
+to be the corresponding header value as a string."
+  (with-input-from-sequence (stream (map 'list 'char-code content-type-header))
+    (with-character-stream-semantics
+     (let* ((*current-error-message* "Corrupted Content-Type header:")
+            (type (read-token stream))
+            (subtype (if (eql #\/ (read-char* stream nil))
+                       (read-token stream)
+                       (return-from parse-content-type
+                         ;; try to return something meaningful
+                         (values "application" "octet-stream" nil))))
+            (parameters (read-name-value-pairs stream))
+            (charset (cdr (assoc "charset" parameters :test #'string=)))
+            (charset
+             (when (string-equal type "text")
+               charset)))
+       (values type subtype charset)))))
 
-(defun get-token-and-parameters (header)
-  (with-input-from-string (stream header)
-    (let* ((*current-error-message* (format nil "Corrupted header ~S:" header))
-           (token (read-token stream))
-           (parameters (read-name-value-pairs stream)))
-      (values token parameters))))
-
-(defun keep-alive-p ()
-  "Returns a true value unless the incoming request obviates a
-keep-alive reply.  The second return value denotes whether the client
-has explicitly asked for a persistent connection."
+(defun keep-alive-p (request)
+  "Returns a true value unless the incoming request's headers or the
+server's PERSISTENT-CONNECTIONS-P setting obviate a keep-alive reply.
+The second return value denotes whether the client has explicitly
+asked for a persistent connection."
   (let ((connection-values
          ;; the header might consist of different values separated by commas
-         (when-let (connection-header (header-in :connection))
+         (when-let (connection-header (header-in :connection request))
            (split "\\s*,\\s*" connection-header))))
     (flet ((connection-value-p (value)
              "Checks whether the string VALUE is one of the
 values of the `Connection' header."
              (member value connection-values :test #'string-equal)))
       (let ((keep-alive-requested-p (connection-value-p "keep-alive")))
-        (values (and (or (and (eq (server-protocol) :http/1.1)
+        (values (and (acceptor-persistent-connections-p *acceptor*)
+                     (or (and (eq (server-protocol request) :http/1.1)
                               (not (connection-value-p "close")))
-                         (and (eq (server-protocol) :http/1.0)
+                         (and (eq (server-protocol request) :http/1.0)
                               keep-alive-requested-p)))
                 keep-alive-requested-p)))))
 
 (defun address-string ()
   "Returns a string with information about Hunchentoot suitable for
 inclusion in HTML output."
-  (format nil "<address>~:[~3*~;<a href='http://httpd.apache.org/'>~A</a> / <a href='http://www.fractalconcept.com/asp/html/mod_lisp.html'>mod_lisp~A~@[/~A~]</a> / ~]<a href='http://weitz.de/hunchentoot/'>Hunchentoot ~A</a> <a href='~A'>(~A ~A)</a>~@[ at ~A~:[ (port ~D)~;~]~]</address>"
-          (server-mod-lisp-p *server*)
-          (or (header-in :server-baseversion) "Apache")
-          (or (header-in :modlisp-major-version) "")
-          (header-in :modlisp-version)
+  (format nil "<address><a href='http://weitz.de/hunchentoot/'>Hunchentoot ~A</a> <a href='~A'>(~A ~A)</a>~@[ at ~A~:[ (port ~D)~;~]~]</address>"
           *hunchentoot-version*
           +implementation-link+
           (escape-for-html (lisp-implementation-type))
           (escape-for-html (lisp-implementation-version))
-          (or (host *request*) (server-address *server*))
+          (or (host *request*) (acceptor-address *acceptor*))
           (scan ":\\d+$" (or (host *request*) ""))
-          (server-port)))
+          (acceptor-port *acceptor*)))
 
 (defun server-name-header ()
   "Returns a string which can be used for 'Server' headers."
   (format nil "Hunchentoot ~A" *hunchentoot-version*))
 
 (defun input-chunking-p ()
-  "Whether input chunking is currently switched on for \(the socket
-stream underlying) *HUNCHENTOOT-STREAM* - note that this will return
-NIL if the underlying stream of the flexi stream is not a chunked
-stream."
-  (chunked-stream-input-chunking-p (flexi-stream-stream *hunchentoot-stream*)))
+  "Whether input chunking is currently switched on for
+*HUNCHENTOOT-STREAM* - note that this will return NIL if the stream
+not a chunked stream."
+  (chunked-stream-input-chunking-p *hunchentoot-stream*))
+
+(defun ssl-p (&optional (acceptor *acceptor*))
+  "Whether the current connection to the client is secure."
+  (acceptor-ssl-p acceptor))
 
-(defun cleanup-function ()
-  "The default for *CLEANUP-FUNCTION*.  Invokes a GC on 32-bit
-LispWorks and does nothing on other Lisps."
-  #+(and :lispworks (not :lispworks-64bit))
-  (hcl:mark-and-sweep 2))
